#!/bin/bash
accumsize=$(($1))
echo
if [ "$accumsize" -lt "9" ]; then
	echo Aborted!  Size range 9-64.
	echo
	exit
fi

double=$(( $accumsize*2 ))

scoredir=test-s${accumsize}

printf "PractRand scoring selected combinations of Xorshift${double}+ and Xoroshiro${double}+ random generators.\n" | tee "${scoredir}/PRscore-xs-s${accumsize}.log" | tee "${scoredir}/PRscore-xo-s${accumsize}.log"

# Combine all earlier findings into a single file
#cat "${scoredir}/maxiter-xo-s${accumsize}-"* >"${scoredir}/maxiter-xo-s${accumsize}.out"
#cat "${scoredir}/maxiter-xs-s${accumsize}-"* >"${scoredir}/maxiter-xs-s${accumsize}.out"

if [[ $? != 0 ]]; then
	echo Aborted!  Nothing done.
	echo
	exit
fi

# Check for spare mips - initial sample point
read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
activeafter=$(( $user + $nice + $system + $softirq + $steal ))
totalafter=$(( $activeafter + $idle + $iowait ))


# Xorshift - Read one line at a time from the file, picking out the first three fields
while IFS=", " read -n 9 a b c
do
	read line
#echo $line
#echo $p1$p2$p3
#exit

# Extract fields from each line
#	a=$(echo $line | awk -F'[, \t]*' '{print $1}')
#	b=$(echo $line | awk -F'[, \t]*' '{print $2}')
#	c=$(echo $line | awk -F'[, \t]*' '{print $3}')
#echo $a$b$c

# Convert to integers
	a=$(($a))
	b=$(($b))
	c=$(($c))

	printf "Scoring Xorshift${double}+ combination [${a} ${b} ${c}] at " | tee -a "${scoredir}/PRscore-xs-s${accumsize}.log"
	date +"%F %T" | tee -a "${scoredir}/PRscore-xs-s${accumsize}.log"

# Variants:  Full sized generator word (minus the lsb for quality), truncated byte (adjustable position), truncated bit (very slow).
	nice -11 src/testcase-xorshift.sh ${accumsize} ${a} ${b} ${c} "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
	if [ "${accumsize}" -lt "21" ]; then
		nice -11 src/bytecase-xorshift.sh ${accumsize} ${a} ${b} ${c} $((${accumsize}-8)) "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
	fi
	if [ "${accumsize}" -gt "11" ] && [ "${accumsize}" -lt "21" ]; then
		nice -11 src/bytecase-xorshift.sh ${accumsize} ${a} ${b} ${c} $(((${accumsize}-8)/2)) "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
#		nice -11 src/bytecase-xorshift.sh ${accumsize} ${a} ${b} ${c} 1 "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
	fi
	if [ "${accumsize}" -lt "19" ]; then
		nice -11 src/bitcase-xorshift.sh ${accumsize} ${a} ${b} ${c} "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
	fi

# Check for spare mips
	while true
	do
		activebefore=$activeafter
		totalbefore=$totalafter
		sleep 3
		read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
		activeafter=$(( $user + $nice + $system + $softirq + $steal ))
		totalafter=$(( $activeafter + $idle + $iowait ))

		cpuload=$(( 101 * ($activeafter - $activebefore) / ($totalafter - $totalbefore) ))
#echo cpuload=${cpuload}
		if [ "$cpuload" -lt "70" ];  then
			break
		fi
	done

done < "${scoredir}/maxiter-xs-s${accumsize}.out"


# Xoroshiro - Read one line at a time from the file, picking out the first three fields
while IFS=", " read -n 9 a b c
do
	read line
# Convert to integers
	a=$(($a))
	b=$(($b))
	c=$(($c))

	printf "Scoring Xoroshiro${double}+ combination [${a} ${b} ${c}] at " | tee -a "${scoredir}/PRscore-xo-s${accumsize}.log"
	date +"%F %T" | tee -a "${scoredir}/PRscore-xo-s${accumsize}.log"

# Variants:  Full sized generator word (minus the lsb for quality), truncated byte (adjustable position), truncated bit (not used, too slow).
	nice -11 src/testcase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
	if [ "${accumsize}" -lt "21" ]; then
		nice -11 src/bytecase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} $((${accumsize}-8)) "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
	fi
	if [ "${accumsize}" -gt "11" ] && [ "${accumsize}" -lt "21" ]; then
		nice -11 src/bytecase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} $(((${accumsize}-8)/2)) "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
#		nice -11 src/bytecase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} 1 "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
	fi
	if [ "${accumsize}" -gt "11" ] && [ "${accumsize}" -lt "19" ]; then
		nice -11 src/bitcase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
	fi

# Check for spare mips
	while true
	do
		activebefore=$activeafter
		totalbefore=$totalafter
		sleep 3
		read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
		activeafter=$(( $user + $nice + $system + $softirq + $steal ))
		totalafter=$(( $activeafter + $idle + $iowait ))

		cpuload=$(( 101 * ($activeafter - $activebefore) / ($totalafter - $totalbefore) ))
#echo cpuload=${cpuload}
		if [ "$cpuload" -lt "70" ];  then
			break
		fi
	done

done < "${scoredir}/maxiter-xo-s${accumsize}.out"
